@@ -52,6 +52,9 @@ typedef struct {
 	unsigned int red;	/* table reduction level (from 1 to 4)*/
 	unsigned int longtable;	/* use package longtable */
 	unsigned int escape;	/* escape TeX control chars or not */
+	unsigned int repeat;	/* repeat table headers for each LaTeX table section or not */
+	unsigned int vlines;	/* insert vertical lines between columns or not */
+	unsigned int hlines;	/* insert horizontal lines between rows or not */
 	char* clrrow;	/* row graylevel (from 0 to 1) */
 	texcape* tex;	/* TeX escapes */
 } config;
@@ -71,6 +74,9 @@ void rtfm(char* prog) {
    printf("	[--position <l|c|r>] (LaTeX) text align in cells\n");
    printf("	[--colorrows graylevel] (LaTeX) alternate gray rows (e.g: 0.75)\n");
    printf("	[--reduce level] (LaTeX) reduce table size (e.g: 1)\n");
+   printf("	[--repeatheader] (LaTeX) repeat table header (for long tables)\n");
+   printf("	[--nohlines] (LaTeX) don't put hline between table rows\n");
+   printf("	[--novlines] (LaTeX) don't put vline between columns\n");
    printf("		csv_file.csv\n");
    printf("The \"longtable\" option needs the {longtable} LaTeX package\n");
    printf("The \"colorrows\" option needs the {colortbl} LaTeX package\n");
@@ -87,25 +93,28 @@ config* parseOptions (config* conf, int argc, char **argv) {
 #else
    int longopt_index = 0;
    static struct option long_options[] = {
-		{"help",      0, NULL, 'h'},
-		{"guess",     0, NULL, 'g'},
-		{"block",     1, NULL, 'b'},
-		{"lines",     1, NULL, 'l'},
-		{"noescape",  0, NULL, 'x'},
-		{"nohead",    0, NULL, 'n'},
-		{"version",   0, NULL, 'v'},
-		{"position",  1, NULL, 'p'},
-		{"separator", 1, NULL, 's'},
-		{"colorrows", 1, NULL, 'c'},
-		{"reduce",    1, NULL, 'r'},
-		{"longtable", 0, NULL, 't'},
+		{"help",      	0, NULL, 'h'},
+		{"guess",     	0, NULL, 'g'},
+		{"block",     	1, NULL, 'b'},
+		{"lines",     	1, NULL, 'l'},
+		{"noescape",  	0, NULL, 'x'},
+		{"nohead",    	0, NULL, 'n'},
+		{"version",   	0, NULL, 'v'},
+		{"position",  	1, NULL, 'p'},
+		{"separator", 	1, NULL, 's'},
+		{"colorrows", 	1, NULL, 'c'},
+		{"reduce",    	1, NULL, 'r'},
+		{"longtable", 	0, NULL, 't'},
+		{"repeatheader",0, NULL, 'e'},
+		{"novlines",		0, NULL, 'y'},
+		{"nohlines",		0, NULL, 'z'},
 		{NULL, 0, NULL, 0} /* marks end-of-list */
 	};
 #endif
 #if defined USE_GETOPT
-	while ((opt = getopt (argc, argv, "hvgnxt?b:l:p:s:c:r:")) != EOF) {
+	while ((opt = getopt (argc, argv, "hvgnxteyz?b:l:p:s:c:r:")) != EOF) {
 #else
-	while ((opt = getopt_long (argc, argv, "hvgnxt?b:l:p:s:c:r:", long_options, &longopt_index)) > 0) {
+	while ((opt = getopt_long (argc, argv, "hvgnxteyz?b:l:p:s:c:r:", long_options, &longopt_index)) > 0) {
 #endif
 		switch (opt) {
 			case '?':
@@ -116,7 +125,7 @@ config* parseOptions (config* conf, int argc, char **argv) {
 			case 'g': /* guess the CSV */
 					conf->guess=1;
 					break;
-			case 't': /* use package longtable */ /* thanks to <Christof.Bodner@infineon.com> */
+			case 't': /* use package longtable */ /* thx to <Christof.Bodner@infineon.com> */
 					conf->longtable=1;
 					break;
 			case 'b': /* csv block delimiter */
@@ -183,6 +192,15 @@ config* parseOptions (config* conf, int argc, char **argv) {
 						exit(EXIT_FAILURE);
 					}
 					break;
+			case 'e': /*repeat table header for each table section*/
+					conf->repeat=1;
+					break;
+			case 'y': /*don't draw vlines between columns*/
+					conf->vlines=0;
+					break;
+			case 'z': /*don't draw hlines between rows*/
+					conf->hlines=0;
+					break;
 		}
 	}
 	return conf;
@@ -292,9 +310,15 @@ void doTeXsub(config* conf, char newsep, FILE* in, FILE* out) {
 	int token=0;
 	int max;
 	int numcols;
-	int lines;
+	unsigned int lines;
 	int inblock=0;
 	int csvrows;
+	int firstrow=1;
+	int nosep=0;
+	int token1=0;
+	int token2=0;
+	char headerrow[1000];
+	headerrow[0]='\0';
 
 	max=numcols=conf->cols;
 	csvrows=conf->rows;
@@ -302,6 +326,8 @@ void doTeXsub(config* conf, char newsep, FILE* in, FILE* out) {
 	lines=(conf->lines)?conf->lines:MAXUINT;
 
 	while (token!=EOF) {
+		token2=token1;	/* second last character, used for detection of quotation marks */
+		token1=token;	/* last character, used for detection of quotation marks */
 		token=getc(in);
 		
 		/* EOF ? */
@@ -317,39 +343,80 @@ void doTeXsub(config* conf, char newsep, FILE* in, FILE* out) {
 				putc(newsep,out);
 				numcols--;
 			}
-			fprintf(out,"\\\\\n"); /* TeX new line */
-			fprintf(out,"\\hline\n"); /* TeX draw hline */
+			if (!(firstrow && (conf->longtable && conf->repeat))) {
+				fprintf(out,"\\\\\n"); /* TeX new line */
+			} else { /* first row and repeat and longtable */
+				fprintf(out, "%s\\\\\n", headerrow);
+			}
+			if(conf->hlines) {
+				fprintf(out,"\\hline\n"); /* TeX draw hline */
+			}
+			if (firstrow && (conf->longtable && conf->repeat)) {
+				fprintf(out, "%s", "\\endhead\n");
+			}
+			if(firstrow && conf->repeat) {
+				char tmp[12];
+				sprintf(tmp,(conf->hlines?"\\\\\n\\hline":"\\\\\n"));
+				strcat(headerrow,tmp);
+			}
+			firstrow=0;
 			numcols=max; /* reset numcols */
 			lines--;
 			csvrows--;
 			/* put a colored row or not (alternate) */
-			if (conf->clrrow && (lines % 2)){
+			if (conf->clrrow && (lines % 2)) {
 				fprintf(out,"\\colorrow ");
 			}
-			/* if the LaTeX tabular is full create a new one, except if no more row */
-			if (!lines && csvrows){
-				if (conf->longtable) {
-				  fprintf(out,"\\end{longtable}\n");
-				  fprintf(out, "\\newline\n");
-				  fprintf(out,"\\begin{longtable}{|\n"); }
-				else {
-				  fprintf(out,"\\end{tabular}\n");
-				  fprintf(out, "\\newline\n");
-				  fprintf(out,"\\begin{tabular}{|"); }
-				while(numcols--)
-					fprintf(out, "%c|", conf->pos);
+			/* if the LaTeX tabular is full create a new one, except if no more row or this is a long table */
+			if (!lines && csvrows && !conf->longtable) {
+				fprintf(out,"\\end{tabular}\n");
+				fprintf(out, "\\newline\n");
+				fprintf(out,"\\begin{tabular}{");
+				if(conf->vlines) {
+					putc('|',out);
+				}
+				while(numcols--) {
+					putc(conf->pos, out);
+					if(conf->vlines) {
+						putc('|',out);
+					}
+				}
 				fprintf(out, "}\n");
-				fprintf(out, "\\hline\n");
+				if(conf->hlines) {
+					fprintf(out, "\\hline\n");
+				}
+				if(conf->repeat && !conf->longtable) {
+					fprintf(out,"%s",headerrow);
+					putc('\n',out);
+				}
 				numcols=max;
 				lines=(conf->lines)?conf->lines:MAXUINT;
-			} /* else end of CSV data */
+			}
+				/* else end of CSV data */
 			continue;
 		}
 
+		/* if commas in cells */
+		/* thx to <florian@heinze.at> */
+		if (token == '\"'){
+			if (nosep==0)
+				nosep=1;
+			else 
+				nosep=0;
+		}
+		
 		/* new column ? */
-		if (token == conf->sep && ((conf->block && !inblock) || !conf->block)) {
-			putc(newsep,out);
+		if ((token == conf->sep && ((conf->block && !inblock) || !conf->block)) && nosep == 0) {			
+			if (!(firstrow && (conf->longtable && conf->repeat)))
+				putc(newsep,out);
 			numcols--;
+			if(firstrow && conf->repeat)
+			{
+				char tmp[2];
+				tmp[0]=newsep;
+				tmp[1]='\0';
+				strcat(headerrow,tmp);
+			}
 			continue;
 		}
 
@@ -360,7 +427,7 @@ void doTeXsub(config* conf, char newsep, FILE* in, FILE* out) {
 		}
 
 		/* data ? */
-		if (token != conf->block && ((conf->block && inblock) || !conf->block)) {
+		if ((token != conf->block && ((conf->block && inblock) || !conf->block)) && ((token=='\"' && token1=='\"' && token2=='\"') || token!='\"')) {
 			/* look for special TeX char to escape */
 			/* FIXME: put all that into a subroutine */
 			int i=0;
@@ -370,9 +437,23 @@ void doTeXsub(config* conf, char newsep, FILE* in, FILE* out) {
 						switch (token) {
 							case '\\':
 								fprintf(out, "\\textbackslash{}");
+								if(firstrow && conf->repeat)
+								{
+									char tmp[17];
+									sprintf(tmp,"\\textbackslash{}");
+									strcat(headerrow,tmp);
+								}
 								break;
 							default:
 								fprintf(out, "\\%c", token);
+								if(firstrow && conf->repeat)
+								{
+									char tmp[3];
+									tmp[0]='\\';
+									tmp[1]=token;
+									tmp[2]='\0';
+									strcat(headerrow,tmp);
+								}
 								break;
 						}
 						break; /* there was some escaping */
@@ -380,7 +461,15 @@ void doTeXsub(config* conf, char newsep, FILE* in, FILE* out) {
 				}
 			/* or print raw char */
 			if ( (i >= conf->tex->size) || (!conf->escape) ) {
-				putc(token, out);
+				if (!(firstrow && (conf->longtable && conf->repeat)))
+					putc(token, out); /* do not print the header twice */
+				if(firstrow && conf->repeat)
+				{
+					char tmp[2];
+					tmp[0]=token;
+					tmp[1]='\0';
+					strcat(headerrow,tmp);
+				}
 			}
 			continue;
 		}
@@ -421,13 +510,26 @@ void doTeXdoc(config* conf, FILE* in, FILE* out) {
 		fprintf(out,"\\addtolength\\tabcolsep{-%sem}\n", tabcolsep[conf->red]);
 	}
 	if (conf->longtable)
-	  fprintf(out, "\\begin{longtable}{|");
+	{
+	  fprintf(out, "\\begin{longtable}{");
+	  if(conf->vlines)
+		putc('|',out);
+	}
   	else
-	  fprintf(out, "\\begin{tabular}{|");
+	{
+	  fprintf(out, "\\begin{tabular}{");
+	  if(conf->vlines)
+		putc('|',out);
+	}
 	while(numcols--)
-		fprintf(out, "%c|",conf->pos); /* position in cell */
+	{
+		fprintf(out, "%c",conf->pos); /* position in cell */
+		if(conf->vlines)
+			putc('|',out);
+	}
 	fprintf(out, "}\n");
-	fprintf(out, "\\hline\n");
+	if(conf->hlines)
+		fprintf(out, "\\hline\n");
 	doTeXsub(conf, '&', in, out); /* & is LaTeX separator */
 	if (conf->longtable) {
 	  fprintf(out, "\\end{longtable}\n");
@@ -469,6 +571,9 @@ int main (int argc, char **argv) {
 	conf->clrrow=NULL;   /* default; LaTeX */
 	conf->red=0;         /* default; LaTeX */
 	conf->longtable=0;   /* default; without package longtable */
+	conf->repeat=0;		/* default; do not repeat the header row */
+	conf->vlines=1;		/* default; draw lines between columns */
+	conf->hlines=1;		/* default; draw lines between rows */
 	
 	/* TeX charaters to escape */
 	conf->tex=(texcape*)malloc(sizeof(texcape));
